
[manpage_begin struct::graph n 2.2]
[copyright {2008 Alejandro Eduardo Cruz Paz <vidriloco@gmail.com>}]
[moddesc   {Tcl Data Structures}]
[titledesc {Extension to the graph creation and manipulation package}]
[require Tcl 8.2]
[require snit 2.2]
[require struct::graph [opt 2.2]]
[require struct::list  [opt 1.5]]
[description]
[para]

This is a graph extension to the graph creation and manipulation package included in the tcllib package. This extension
incorporates additional commands to the set of commands provided by the graph creation and manipulation package itself.
Some of these commands correspond to the implementation of well known graph theory algorithms, while others allow to 
obtain useful data from the graph (e.g diameter, radius).

[para]

This extension is implemented using Snit. The commands made available by the extension are Snit's methods, while the 
graph creation and manipulation package commands are [term delegated] to the Snit abstract data type, (where the algorithms are defined) so that when an instance of that abstract data type is created, those [term delegated] commands will be considered as methods of that instance.

[para]

In addition to the aditional set of commands, this extension makes possible to have labeled/weighted graphs. 
This feature is part of the underlying implementation. Is defined in the graph structure package, and it's thus independent of the Snit implementation, which mantains modularity, abstraction, and consistency.

[para]

The command for creating a graph is:

[list_begin definitions]

[call [cmd ::graph] [opt [arg graphName]] \
  [option -weighted]|[option -unweighted]] 
	[opt "[const =]|[const :=]|[const as]|[const deserialize] [arg source]"]]

The command creates a new instance of the snit graph abstract data type with a [term handler] which is the name of this graph. This commands receives two options as arguments. The first [term option] relates to specifying if this graph is going to be a [term weighted] or [term unweighted] graph, however , if this option is ommited, the graph will be [term unweighted].

[example {
  # 
  # ::graph graph1
  # ::graph graph1 -weighted | -unweighted
  # ::graph graph1 = | := | as | deserialize graph2
  # ::graph graph1 -weighted | -unweighted = | := | as | deserialize graph2
  #
  # In the precedent examples, different types of graphs objects are being created and/or copied. 
  # The difference arises depending on the option provided (-weighted|-unweighted), if no
  # one has been provided, then it defaults to unweighted. 
  # If used =:|=|as|deserialize source, as in the last example
  # it will copy the data from a source graph.
  #
  #
  # Supponse we build the following graph, first constructing a new ::graph instance with the 
  # -weighted option set.
  #
  #
  #         ------>  node0  --- ----------2---------> node1
  #       --4--              --1--                   ---   ---
  #     ---                    ------>           --3--       --10--
  #  node2 <----------2-----------  node3  <------ ----2-------- ----> node4
  #     ---                         ---    ---                        ---
  #       --5--                 --8--         --4--               --6--
  #         ------>  node5  <------- <--------1----- --> node6  <------
  #     
  #
  #
}]





[call [cmd graphName] [arg option] [opt [arg "arg arg ..."]]]

[arg Option] and the [arg arg]s determine the exact behavior of the
command.

[call [arg graphName] [method dijkstra] [arg sourcenode] \
                      [option -arcmode] [arg undirected] or \
                      [arg directed] [option -outputformat] \
                      [arg tree] or [arg distances]]
 

Returns the shortest path between every two nodes present in this graph, using the Dijkstra algorithm.
This method receives as arguments a [arg sourcenode], which is the node from which the others node's distances will be calculated, it also receives two optional options together with it's arguments.


Option [option -arcmode] tells the algorithm how to consider each of the arcs it encounters. If [option -arcmode] is set to [arg undirected] then for each node, the weights will be calculated considering the arcs coming out of the node and the arcs entering to the node. If [option -arcmode] is set to [arg directed] then for each node, the weights will be calculated considering the arcs adyacent to each one of them, only in the direction they are oriented.
If [option -arcmode] is not provided, the algorithm will work as if [option -arcmode]
with [arg undirected] had been provided.

Option [option -outputformat] tells the algorithm how it's output will be. If [option -outputformat] is set to [arg tree] then the output will be a list of the type "node list_of_ancestors node list_of_ancestors". If [option -outputformat] is set to [arg distances] then the output will be a list of the nodes, together with it's processed distances from the [arg sourcenode].



[example {
  # 
  # If we were applying this algorithm to the graph1 object from the previous example, it would be done in the    
  # following way:
  #
  # If we want the algorithm to work with the graph as a directed graph, and get an output in tree format, we would
  # use:
  # graph1 dijkstra 'node0' -arcmode directed -outputform tree
  # 
  #
  # If we want the algorithm to work with the graph as an undirected graph. This is specially useful when the
  # algorithm starts on a node which has an outdegree of zero, or when there is a node in the graph in such
  # situation. 
  # graph1 dijkstra 'node5' -outputformat undirected
}]

[call [arg graphName] [method distance] [arg node1] [arg node2]]

Returns the distance between any two nodes ([arg node1] and [arg node2]) in the graph. The distance is the shortest path (weights sum) between those two nodes.

[call [arg graphName] [method eccentricity] [arg node] [option -arcmode] [arg directed] or [arg undirected]]

Returns the eccentricity of a node. The eccentricity is the greatest distance between [arg node] and any other vertex.
Option [option -arcmode] behaves in the same way as in Dijkstra's algorithm.

[call [arg graphName] [method radius] [arg node] [option -arcmode] [arg directed] or [arg undirected]]

Returns the radius of this graph. The radius of a graph is the minimum graph eccentricity of any graph node in a graph. Option [option -arcmode] behaves in the same way as in Dijkstra's algorithm.

[call [arg graphName] [method diameter] [arg node] [option -arcmode] [arg directed] or [arg undirected]]

Returns the diameter of this graph. The diameter of a graph is the maximum graph eccentricity of any graph node in a graph. Option [option -arcmode] behaves in the same way as in Dijkstra's algorithm.

[call [arg graphName] [method prim] [arg node]]

Returns a list of nodes together with the weight of the edge adjacent to each of those nodes, that edge has to be the one with minimum weight. This result is a minimum spanning tree of the graph, using an implementation of the Prim algorithm. This method receives as argument a [arg node], from which the algorithm will be executed. 
[emph Note:] This algorithm only works on undirected graphs.

[example {
  #
  # We can use the same object (graph1), the algorithm will work on it as if it was an undirected graph. We would 
  # call it in the following way: 
  # 
  # graph1 prim 'node0'
  # graph1 prim 'node5'
  #
}]

[call [arg graphName] [method tarjan] [arg node]]

This method is an implementation of the Tarjan algorithm, which finds the strongly connected components of a digraph.
In a strongly connected component, for every two nodes U,V there is always a path from U to V and from V to U.
This methods retrieves a list containing the connected components of the graph, however, if it retrieves an empty list, then there are no strongly connected components. 
If the set of connected components is empty, then the graph is a directed acyclic graph.

[example {
  #
  # The usage is very simple, and again we are using the graph1 object:
  # graph1 tarjan 'node0'
  # graph1 tarjan 'node4'
  #
  # In each case, it will return, as expected a list of strongly connected components of this digraph.
  #
}]

[emph Note:] This algorithm works through the graph in a normal way. In other words, it does not consider the arcs
             as edges, as another algorithm implementations do in this package. This algorithm works with directed
             graphs only.


[call [arg graphName] [method {arc weights}] ]

Retrieves a list of the edges in this graph together with each of it's edges weight.

[call [arg graphName] [method {arc setweight}] [arg arcname] [arg weight]]

This method sets a [arg weight] to [arg arcname]. It will verify the existence of [arg arcname], if it exists
then [arg arcname]'s weight will be set to [arg weight]. Any previous value assigned to [arg arcname] will be
overriden.

[call [arg graphName] [method {arc getweight}] [arg arcname]]

This method retrieves the weight previously assigned to [arg arcname]. If [arg arcname] has not been assigned a weight yet, then it will raise an error.

[call [arg graphName] [method {arc setunweighted}] [arg value]]

This method sets a weight for all of those edges which have been not asigned a weight. If [arg value] no provided, 
then it defaults to zero.

[call [arg graphName] [method {arc isweighted?}]]

This method returns one if the current graph is weighted, zero otherwise.

[call [arg graphName] [method connectedness]]

This method returns a list of lists. Each list in the list corresponds to a partition of the nodeset. If the list
contains more than one list. Thus we can know the number of connected components of this graph, if the graph is 
disconnected or connected. This method works only on undirected graphs.

[call [arg graphName] [method iscutvertex?] [arg node]]

This method checks if after removing the node which is passed as argument the number of connected components of the graph augments, in which case means the node provided is indeed a cutvertex of the graph.

[call [arg graphName] [method isbridge?] [arg arc]]

This method checks if after removing the edge which is passed as argument the number of connected components of the graph augments, in which case means the edge provided is a bridge of the graph.

[list_end]

[section {BUGS, IDEAS, FEEDBACK}]

This is a preliminar API for this package, as bugs need to be fixed, and new features are still to be added. 
It is under revision. Please also report any ideas for enhancements you may have for either package and/or documentation.

[keywords graph digraph]
[keywords edge arc node vertex subgraph neighbour]
[keywords adjacent weighted unweighted component connected directed undirected cutvertex bridge]
[manpage_end]
